#!/bin/bash

OTA_PREFIX="air-update.b1.fw"
FIRMWARE_ENCRYPTED_FILE="${OTA_PREFIX}.enc"

SYMMETRIC_KEY="${OTA_PREFIX}.seed"
SYMMETRIC_KEY_ENC="${SYMMETRIC_KEY}.enc"   

PRIVATE_KEY="~/.ssh/id_rsa"
PUBLIC_KEY="~/.ssh/id_rsa.pub"

SECRETS_DIR="/home/pi/secrets"
DECRYPT_KEY="${SECRETS_DIR}/private_key.pem"
ENCRYPT_KEY="${SECRETS_DIR}/public_key.pem"

WORK_DIR="/home/pi/airfwua"
FIRMWARE_DIR="/home/pi/firmware"

if [ ! -d ${SECRETS_DIR} ]; then   
    # Convert SSH keys to PEM format (if needed)'
    mkdir -p ${SECRETS_DIR}

fi
if [ -f "$PRIVATE_KEY" ]; then
        echo "The file $PRIVATE_KEY exists."
        echo "Keys present"
  else
        echo "The file $PRIVATE_KEY does not exist."
        PRIVATE_KEY="/home/pi/.ssh/id_rsa"

fi
rm  -rf ${DECRYPT_KEY} ${ENCRYPT_KEY}
if [ ! -f ${DECRYPT_KEY} ]; then
    openssl pkcs8 -topk8 -nocrypt -in ${PRIVATE_KEY} -outform pem -out ${DECRYPT_KEY}
    openssl rsa -pubout -in ${DECRYPT_KEY} -out ${ENCRYPT_KEY}
else
  echo "Keys present"
fi



# Change to the script directory
script_dir="$( cd "$( dirname "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )"
pushd $script_dir > /dev/null
echo "Changed working directory to $(pwd)"

# Decrypt the symmetric key with the private key
openssl pkeyutl -decrypt -inkey ${DECRYPT_KEY} -in ${SYMMETRIC_KEY_ENC} -out ${SYMMETRIC_KEY}
if [ $? -ne 0 ]; then
    echo "Failed to decrypt symmetric key"
    exit 1
fi
echo "Decrypted symmetric key"

# Decrypt the firmware update package with the symmetric key
openssl enc -d -aes-256-cbc -in ${FIRMWARE_ENCRYPTED_FILE} -out ${OTA_PREFIX} -k $(cat ${SYMMETRIC_KEY})
if [ $? -ne 0 ]; then
    echo "Failed to decrypt package"
    exit 1
fi
echo "Decrypted package"

# Extract the package
mkdir -p ${WORK_DIR}
unzip -oq "${OTA_PREFIX}" -d "${WORK_DIR}"
if [ $? -ne 0 ]; then
    echo "Failed to extract package"
    exit 1
fi
echo "Extracted package"

echo "Moving files to firmware directory"
# Move all the stuff back to /home/pi/firmware but rename the install script
mv ${WORK_DIR}/fwupdate.sh ${FIRMWARE_DIR}/install.sh
chmod 777 ${FIRMWARE_DIR}/install.sh
cp -rf ${WORK_DIR}/* ${FIRMWARE_DIR}
rm -rf ${WORK_DIR}
echo "Moved files to firmware directory"

# Clean up
rm -rf ${FIRMWARE_ENCRYPTED_FILE} ${SYMMETRIC_KEY}  ${SYMMETRIC_KEY_ENC} ${OTA_PREFIX} ./*.sig
echo "Cleaned up"

#Run OTA update script
echo "Running OTA update script"
source ${FIRMWARE_DIR}/install.sh
echo "Running update script"
if [ $? -ne 0 ]; then
    echo "OTA update script exits with non-zero code"
    exit 1
fi
popd > /dev/null



echo "Decryption successful"
exit 0